;----------------------------------------
; PNGLIB module definition file for OS/2
;----------------------------------------

LIBRARY		PNG
DESCRIPTION	"PNG image compression library for OS/2"
CODE		PRELOAD MOVEABLE DISCARDABLE
DATA		PRELOAD MOVEABLE MULTIPLE

EXPORTS
  png_set_sig_bytes
  png_sig_cmp
  png_check_sig
  png_create_read_struct
; png_create_read_struct_2
  png_create_write_struct
; png_create_write_struct_2
  png_write_chunk
  png_write_chunk_start
  png_write_chunk_data
  png_write_chunk_end
  png_create_info_struct
  png_info_init
  png_write_info
  png_read_info
  png_convert_to_rfc1123
  png_convert_from_struct_tm
  png_convert_from_time_t
  png_set_expand
  png_set_bgr
  png_set_gray_to_rgb
; png_set_rgb_to_gray
  png_build_grayscale_palette
  png_set_strip_alpha
  png_set_swap_alpha
  png_set_invert_alpha
  png_set_filler
  png_set_swap
  png_set_packing
  png_set_packswap
  png_set_shift
  png_set_interlace_handling
  png_set_invert_mono
  png_set_background
  png_set_strip_16
  png_set_dither
  png_set_gamma
  png_set_flush
  png_write_flush
  png_start_read_image
  png_read_update_info
  png_read_rows
  png_read_row
  png_read_image
  png_write_row
  png_write_rows
  png_write_image
  png_write_end
  png_read_end
  png_destroy_info_struct
  png_destroy_read_struct
  png_read_destroy
  png_destroy_write_struct
; png_write_destroy_info
  png_write_destroy
  png_set_crc_action
  png_set_filter
  png_set_filter_heuristics
  png_set_compression_level
  png_set_compression_mem_level
  png_set_compression_strategy
  png_set_compression_window_bits
  png_set_compression_method
  png_init_io
  png_set_error_fn
  png_get_error_ptr
; png_get_mem_fn
; png_set_mem_fn
  png_set_write_fn
  png_set_read_fn
  png_set_write_status_fn
  png_set_read_status_fn
  png_get_io_ptr
  png_set_progressive_read_fn
  png_set_read_status_fn
  png_set_read_user_transform_fn
  png_set_write_status_fn
  png_set_write_user_transform_fn
  png_get_progressive_ptr
  png_process_data
  png_progressive_combine_row
  png_malloc
  png_free
  png_memcpy_check
  png_memset_check
; png_malloc_default
; png_free_default
; png_far_to_near
  png_error
  png_chunk_error
  png_warning
  png_chunk_warning
  png_get_valid
  png_get_rowbytes
  png_get_channels
  png_get_image_width
  png_get_image_height
  png_get_bit_depth
  png_get_color_type
  png_get_filter_type
  png_get_interlace_type
  png_get_compression_type
  png_get_pixels_per_meter
  png_get_pixel_aspect_ratio
  png_get_x_offset_pixels
  png_get_y_offset_pixels
  png_get_x_offset_microns
  png_get_y_offset_microns
  png_get_signature
  png_get_bKGD
  png_set_bKGD
  png_get_cHRM
  png_set_cHRM
  png_get_gAMA
  png_set_gAMA
  png_get_hIST
  png_set_hIST
  png_get_IHDR
  png_set_IHDR
  png_get_oFFs
  png_set_oFFs
  png_get_pCAL
  png_set_pCAL
  png_get_pHYs
  png_set_pHYs
  png_get_PLTE
  png_set_PLTE
  png_get_sBIT
  png_set_sBIT
  png_get_sRGB
  png_set_sRGB
  png_set_sRGB_gAMA_and_cHRM
  png_get_text
  png_set_text
  png_get_tIME
  png_set_tIME
  png_get_tRNS
  png_set_tRNS

  png_create_struct
  png_destroy_struct
  png_info_destroy
  png_zalloc
  png_zfree
  png_reset_crc
  png_write_data
  png_read_data
  png_crc_read
  png_crc_finish
  png_crc_error
  png_calculate_crc
  png_flush
  png_save_uint_32
  png_save_int_32
  png_save_uint_16
  png_write_sig
  png_write_IHDR
  png_write_PLTE
  png_write_IDAT
  png_write_IEND
  png_write_gAMA
  png_write_sBIT
  png_write_cHRM
  png_write_sRGB
  png_write_tRNS
  png_write_bKGD
  png_write_hIST
  png_check_keyword
  png_write_tEXt
  png_write_zTXt
  png_write_oFFs
  png_write_pCAL
  png_write_pHYs
  png_write_tIME
  png_write_finish_row
  png_write_start_row
  png_build_gamma_table
  png_combine_row
  png_do_read_interlace
  png_do_write_interlace
  png_read_filter_row
  png_write_find_filter
  png_write_filtered_row
  png_read_finish_row
  png_read_start_row
  png_read_transform_info
  png_do_read_filler
  png_do_read_swap_alpha
  png_do_write_swap_alpha
  png_do_read_invert_alpha
  png_do_write_invert_alpha
  png_do_strip_filler
  png_do_swap
  png_do_packswap
; png_do_rgb_to_gray
  png_do_gray_to_rgb
  png_do_unpack
  png_do_unshift
  png_do_invert
  png_do_chop
  png_do_dither
; png_correct_palette
  png_do_bgr
  png_do_pack
  png_do_shift
  png_do_background
  png_do_gamma
  png_do_expand_palette
  png_do_expand
  png_handle_IHDR
  png_handle_PLTE
  png_handle_IEND
  png_handle_gAMA
  png_handle_sBIT
  png_handle_cHRM
  png_handle_sRGB
  png_handle_tRNS
  png_handle_bKGD
  png_handle_hIST
  png_handle_oFFs
  png_handle_pCAL
  png_handle_pHYs
  png_handle_tIME
  png_handle_tEXt
  png_handle_zTXt
  png_handle_unknown
  png_check_chunk_name
  png_do_read_transformations
  png_do_write_transformations
  png_init_read_transformations
  png_push_read_chunk
  png_push_read_sig
; png_push_check_crc
  png_push_crc_skip
  png_push_crc_finish
  png_push_fill_buffer
  png_push_save_buffer
  png_push_restore_buffer
  png_push_read_IDAT
  png_process_IDAT_data
  png_push_process_row
  png_push_handle_unknown
  png_push_have_info
  png_push_have_end
  png_push_have_row
; png_push_read_end
  png_process_some_data
  png_read_push_finish_row
  png_push_handle_tEXt
  png_push_read_tEXt
  png_push_handle_zTXt
  png_push_read_zTXt

  png_libpng_ver
  png_pass_start
  png_pass_inc
  png_pass_ystart
  png_pass_yinc
  png_pass_mask
  png_pass_dsp_mask
; png_pass_width
; png_pass_height

  png_IHDR
  png_IDAT
  png_IEND
  png_PLTE
  png_bKGD
  png_cHRM
  png_gAMA
  png_hIST
  png_oFFs
  png_pCAL
  png_pHYs
  png_sBIT
  png_sRGB
  png_tEXt
  png_tIME
  png_tRNS
  png_zTXt
